#include "i2c.h"
#include <avr/io.h>

void i2c_init() {
  // Set SCL to 400kHz
  TWSR = 0;  // prescaler = 1
  TWBR = 12; // SCL frequency = CPU clock / (16 + 2 * TWBR * prescaler) = 400kHz
}

uint8_t i2c_wait() {
  while (!(TWCR & (1 << TWINT)))
    ;

  return TWSR & 0xF8;
}

uint8_t i2c_start() {
  TWCR = (1 << TWINT) | (1 << TWSTA) | (1 << TWEN);
  return i2c_wait();
}

uint8_t i2c_stop() { TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO); }

uint8_t i2c_send(uint8_t data) {
  TWDR = data;
  TWCR = (1 << TWINT) | (1 << TWEN);
  return i2c_wait();
}

uint8_t i2c_receive(uint8_t ack) {
  if (ack) {
    TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWEA);
  } else {
    TWCR = (1 << TWINT) | (1 << TWEN);
  }
  i2c_wait();
  return TWDR;
}

void i2c_transfer(uint8_t addr, struct i2c_message *messages, uint8_t size) {
  for (int k = 0; k < size; k++) {
    i2c_start();

    uint8_t address_r = (addr << 1) | (messages[k].read ? 1 : 0);
    i2c_send(address_r);

    for (int j = 0; j < messages[k].size; j++) {
      if (messages[k].read) {
        messages[k].data[j] = i2c_receive(j != messages[k].size - 1);
      } else {
        i2c_send(messages[k].data[j]);
      }
    }
  }

  i2c_stop();
}
