#include "mma8451.h"
#include "uart.h"
#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdint.h>
#include <stdio.h>
#include <util/delay.h>

// Données mesurées
volatile struct mma8451_data data;

// Temps écoulé entre les deux dernières interruptions
uint32_t time_elapsed = 0;

ISR(INT0_vect) {
  // Stockage du temps écoulé en µs
  time_elapsed = TCNT1 * 4;
  TCNT1 = 0;

  // Lecture des données
  sei();
  data = mma8451_read();
}

int main() {
  // Initialisation de l'UART
  uart_init(115200);

  // Initialisation du MMA8451 avec les interruptions activées
  mma8451_init(1);

  // Paramétrage de TIMER1 avec 4µs par tick
  TCCR1B = (1 << CS11) | (1 << CS10); // Prescaler à 64
  TCNT1 = 0;                          // Compteur à 0

  // Active l'interrupt sur les fronts descendants (INT0)
  EICRA = (1 << ISC01);
  EIMSK |= (1 << INT0);

  // Active les interruptions
  sei();

  while (1) {
    char buffer[100];

    // Affichage des données
    sprintf(buffer, "x: %d, y: %d, z: %d\r\n", data.x, data.y, data.z);
    uart_send_string(buffer);

    // Affichage du temps écoulé
    sprintf(buffer, "duree mesure: %lu us\r\n", time_elapsed);
    uart_send_string(buffer);

    _delay_ms(100);
  }
}
