#include "mma8451.h"
#include "i2c.h"
#include "uart.h"
#include <avr/io.h>

// Lis un octet dans les registres de l'accéléromètre
uint8_t mma8451_read_byte(uint8_t register_addr) {
  struct i2c_message messages[2] = {
      {.read = 0, .data = &register_addr, .size = 1},
      {.read = 1, .data = 0, .size = 1},
  };

  i2c_transfer(MMA8451_I2C_ADDRESS, messages, 2);

  return messages[1].data[0];
}

// Écrit un octet dans les registres de l'acceleromètre à l'adresse donnée
void mma8451_write_byte(uint8_t register_addr, uint8_t value) {
  char data[2] = {register_addr, value};
  struct i2c_message messages[1] = {
      {.read = 0, .data = data, .size = 2},
  };

  i2c_transfer(MMA8451_I2C_ADDRESS, messages, 1);
}

// Réalise une mesure
struct mma8451_data mma8451_read() {
  struct mma8451_data data;
  uint8_t raw_data[6];

  // Adresse de départ
  uint8_t start_register = MMA8451_OUT_X_MSB;

  // Les deux messages qui seront échangés (la demande, puis la réponse du
  // périphérique)
  struct i2c_message messages[2] = {
      {.read = 0, .data = &start_register, .size = 1},
      {.read = 1, .data = raw_data, .size = 6},
  };

  // Réalise le transfert
  i2c_transfer(MMA8451_I2C_ADDRESS, messages, 2);

  // Décodage des données du message
  data.x = ((raw_data[0] << 8) | raw_data[1]) >> 2;
  data.y = ((raw_data[2] << 8) | raw_data[3]) >> 2;
  data.z = ((raw_data[4] << 8) | raw_data[5]) >> 2;

  return data;
}

// Initialise le périphérique MMA8451
void mma8451_init(uint8_t enable_interrupts) {
  i2c_init();

  if (enable_interrupts) {
    // Reset en mode standby pour pouvoir écrire dans les registres de contrôle
    mma8451_write_byte(MMA8451_CTRL_REG1, 0x00);

    // Active l'interruption DRDY sur INT1
    mma8451_write_byte(MMA8451_CTRL_REG4, 0x01);
    mma8451_write_byte(MMA8451_CTRL_REG5, 0x01);
  }

  // Démarrage des mesures à 100Hz
  uint8_t ctrl = (1 << CTRL_REG1_ACTIVE) | (0b11 << CTRL_REG1_DR0);
  mma8451_write_byte(MMA8451_CTRL_REG1, ctrl);
}
